@@ -5,11 +5,15 @@ The package was developed to serve two purposes:
     2. provide succinct workflows of those tools to automate the process of intaking climate data and morphing an EPW file
 The second task is dealt with throughout the following scripts
 """
+import copy
 import datetime
+import os
 
 from pyepwmorph.models import access, coordinate, assemble
 from pyepwmorph.morph import procedures
 from pyepwmorph.tools import io as morph_io
+from pyepwmorph.tools import utilities as morph_utils
+from pyepwmorph.tools import configuration as morph_config
 import warnings
 
 warnings.filterwarnings("ignore")
@@ -41,8 +45,8 @@ def compile_climate_model_data(model_sources, pathway, variable, longitude, lati
         longitude (-180 to 180)
     latitude : float
         latitude (-180 to 180)
-    percentiles : list
-        percentiles from which to slice the ensemble of data
+    percentiles : list of ints
+        percentiles from which to slice the ensemble of data such as [10,50] or [95]
 
     Returns
     -------
@@ -116,7 +120,7 @@ def morph_epw(epw_file, user_variables, baseline_range, future_range, model_data
     future_range : tuple (int,int)
         the start and stop for the future dates to capture the climaatology for
     model_data_dict : dict
-        a dict of dicts of pandas dataframes where pathway is the first key is the pathway and the second key is variable
+        a dict of dicts of pandas dataframes where pathway is the first key and the second key is variable
     pathway : string
         the pathway that is being is being selected from the model_data_dict - from ['historical','ssp126','ssp245','ssp585']
     percentile : int
@@ -132,7 +136,10 @@ def morph_epw(epw_file, user_variables, baseline_range, future_range, model_data
     # iterate through each if variable statement
     # add to blank dataframe using column names of existing epw
     # at end replace columns in existing with columns in new by iterating through columns in new
-    epw_object = morph_io.Epw(epw_file)
+    if type(epw_file) == str:
+        epw_object = morph_io.Epw(epw_file)
+    else:
+        epw_object = copy.deepcopy(epw_file)
 
     morphed_dict = {}
 
@@ -176,9 +183,67 @@ def morph_epw(epw_file, user_variables, baseline_range, future_range, model_data
                                                                          'huss')
 
                 present_relhum = epw_object.dataframe['relhum_percent']
-                morphed_relhum = procedures.morph_relhum(present_relhum,
-                                                         relhum_climatologies[1], relhum_climatologies[0]
-                                                         ).values
+                present_psl = epw_object.dataframe['atmos_Pa']
+                present_dbt = epw_object.dataframe['drybulb_C']
+
+                if ('atmos_Pa' in morphed_dict.keys()) & ('drybulb_C' in morphed_dict.keys()):
+                    morphed_relhum = procedures.morph_relhum(present_relhum,
+                                                             present_psl, present_dbt,
+                                                             morphed_dict['atmos_Pa'], morphed_dict['drybulb_C'],
+                                                             relhum_climatologies[1], relhum_climatologies[0]
+                                                             ).values
+                else:
+                    # TODO add a catch here if the tas, tasmax, tasmin, and pressure variables have not been downlaoded yet
+                    # download them here so no error arises
+
+
+                    # morph temp
+                    tas_climatologies = assemble.calc_model_climatologies(baseline_range, future_range,
+                                                                          model_data_dict['historical']['tas'][
+                                                                              percentile],
+                                                                          model_data_dict[pathway]['tas'][percentile],
+                                                                          'tas')
+                    tmax_climatologies = assemble.calc_model_climatologies(baseline_range, future_range,
+                                                                           model_data_dict['historical']['tasmax'][
+                                                                               percentile],
+                                                                           model_data_dict[pathway]['tasmax'][
+                                                                               percentile],
+                                                                           'tasmax')
+                    tmin_climatologies = assemble.calc_model_climatologies(baseline_range, future_range,
+                                                                           model_data_dict['historical']['tasmin'][
+                                                                               percentile],
+                                                                           model_data_dict[pathway]['tasmin'][
+                                                                               percentile],
+                                                                           'tasmin')
+                    present_dbt = epw_object.dataframe['drybulb_C']
+                    morphed_dbt = procedures.morph_dbt_year(present_dbt,
+                                                            tas_climatologies[1], tas_climatologies[0],
+                                                            tmax_climatologies[1], tmax_climatologies[0],
+                                                            tmin_climatologies[1], tmin_climatologies[0]
+                                                            ).values
+                    if 'atmos_Pa' in morphed_dict.keys():
+                        morphed_psl = morphed_dict['atmos_Pa']
+                    else:
+                        # morph psl
+                        psl_climatologies = assemble.calc_model_climatologies(baseline_range, future_range,
+                                                                              model_data_dict['historical']['psl'][
+                                                                                  percentile],
+                                                                              model_data_dict[pathway]['psl'][
+                                                                                  percentile],
+                                                                              'psl')
+
+                        morphed_psl = procedures.morph_psl(present_psl,
+                                                              psl_climatologies[1], psl_climatologies[0]
+                                                              ).values
+                        if "Pressure" in user_variables:
+                            morphed_dict['atmos_Pa'] = morphed_psl
+
+                    morphed_relhum = procedures.morph_relhum(present_relhum,
+                                                             present_psl, present_dbt,
+                                                             morphed_psl, morphed_dbt,
+                                                             relhum_climatologies[1], relhum_climatologies[0]
+                                                             ).values
+
                 morphed_dict['relhum_percent'] = morphed_relhum
 
         elif variable == 'Dew Point':
@@ -187,7 +252,7 @@ def morph_epw(epw_file, user_variables, baseline_range, future_range, model_data
                 morphed_dict['dewpoint_C'] = morphed_dewpt.values
 
             else:
-                # TODO add a catch here if the tas, tasmax, tasmin, huss variables have not been downlaoded yet
+                # TODO add a catch here if the tas, tasmax, tasmin, huss, and psl variables have not been downlaoded yet
                 # download them here so no error arises
 
                 # morph temp
@@ -220,7 +285,26 @@ def morph_epw(epw_file, user_variables, baseline_range, future_range, model_data
                                                                          'huss')
 
                 present_relhum = epw_object.dataframe['relhum_percent']
+                present_psl = epw_object.dataframe['atmos_Pa']
+
+                if 'atmos_Pa' in morphed_dict.keys():
+                    morphed_psl = morphed_dict['atmos_Pa']
+                else:
+                    psl_climatologies = assemble.calc_model_climatologies(baseline_range, future_range,
+                                                                          model_data_dict['historical']['psl'][
+                                                                              percentile],
+                                                                          model_data_dict[pathway]['psl'][percentile],
+                                                                          'psl')
+                    present_psl = epw_object.dataframe['atmos_Pa']
+                    morphed_psl = procedures.morph_psl(present_psl,
+                                                       psl_climatologies[1], psl_climatologies[0]
+                                                       ).values
+                    if "Pressure" in user_variables:
+                        morphed_dict['atmos_Pa'] = morphed_psl
+
+
                 morphed_relhum = procedures.morph_relhum(present_relhum,
+                                                         present_psl, present_dbt, morphed_psl, morphed_dbt,
                                                          relhum_climatologies[1], relhum_climatologies[0]
                                                          ).values
                 if "Temperature" in user_variables:
@@ -229,6 +313,7 @@ def morph_epw(epw_file, user_variables, baseline_range, future_range, model_data
                 if "Humidity" in user_variables:
                     morphed_dict['relhum_percent'] = morphed_relhum
 
+
                 morphed_dewpt = procedures.morph_dewpt(morphed_dict['drybulb_C'], morphed_dict['relhum_percent'])
                 morphed_dict['dewpoint_C'] = morphed_dewpt.values
 
@@ -239,10 +324,10 @@ def morph_epw(epw_file, user_variables, baseline_range, future_range, model_data
                                                                   'psl')
 
             present_psl = epw_object.dataframe['atmos_Pa']
-            moprhed_psl = procedures.morph_relhum(present_psl,
-                                                  psl_climatologies[1], psl_climatologies[0]
-                                                  ).values
-            morphed_dict['atmos_Pa'] = moprhed_psl
+            morphed_psl = procedures.morph_psl(present_psl,
+                                               psl_climatologies[1], psl_climatologies[0]
+                                               ).values
+            morphed_dict['atmos_Pa'] = morphed_psl
 
         elif variable == 'Wind':
             vas_climatologies = assemble.calc_model_climatologies(baseline_range, future_range,
@@ -255,14 +340,14 @@ def morph_epw(epw_file, user_variables, baseline_range, future_range, model_data
                                                                   'uas')
 
             present_wspd = epw_object.dataframe['windspd_ms']
-            moprhed_wspd = procedures.morph_wspd(present_wspd,
+            morphed_wspd = procedures.morph_wspd(present_wspd,
                                                  vas_climatologies[1], vas_climatologies[0],
                                                  uas_climatologies[1], uas_climatologies[0]
                                                  ).values
-            morphed_dict['windspd_ms'] = moprhed_wspd
+            morphed_dict['windspd_ms'] = morphed_wspd
 
         elif variable == 'Clouds and Radiation':
-            # do morphing for glohor, diffhor, dirnor, tsc, osc
+            # do morphing for glohor, difhor, dirnor, tsc, osc
             longitude = epw_object.location['longitude']
             latitude = epw_object.location['latitude']
             utc_offset = epw_object.location['utc_offset']
@@ -283,11 +368,12 @@ def morph_epw(epw_file, user_variables, baseline_range, future_range, model_data
             morphed_dict['glohorrad_Whm2'] = morphed_glohor
 
             present_exthor = epw_object.dataframe['exthorrad_Whm2']
-            morphed_diffhor = procedures.calc_diffhor(longitude, latitude, utc_offset,
+            morphed_difhor = procedures.calc_difhor(longitude, latitude, utc_offset,
                                                       morphed_glohor, present_exthor
                                                       ).values
-            morphed_dict['diffhorrad_Whm2'] = morphed_diffhor
+            morphed_dict['difhorrad_Whm2'] = morphed_difhor
             morphed_dirnor = procedures.calc_dirnor(morphed_glohor,
+                                                    morphed_difhor,
                                                     longitude, latitude, utc_offset
                                                     ).values
             morphed_dict['dirnorrad_Whm2'] = morphed_dirnor
@@ -310,8 +396,47 @@ def morph_epw(epw_file, user_variables, baseline_range, future_range, model_data
         epw_object.dataframe[k] = v
         if n == 0:
             epw_object.headers['COMMENTS 2'][
-                0] += f'morphed for {pathway} ({future_range}) with pyepwmorph on {datetime.datetime.now().isoformat()}: {k},'
+                0] += f' morphed for {pathway} ({future_range}) with pyepwmorph on {datetime.datetime.now().isoformat()}: {k},'
         else:
             epw_object.headers['COMMENTS 2'][0] += f'{k},'
 
     return epw_object
+
+
+def morphing_workflow(project_name, epw_file, user_variables, user_pathways, percentiles, future_years, output_directory, model_sources=None, baseline_range=None, write_file=True):
+    config_object = morph_config.MorphConfig(project_name, epw_file, user_variables, user_pathways, percentiles,
+                                             future_years, output_directory, model_sources=model_sources, baseline_range=baseline_range)
+
+    result_data = {}
+
+    # get climate model data
+    year_model_dict = iterate_compile_model_data(config_object.model_pathways,
+                                                                 config_object.model_variables,
+                                                                 config_object.model_sources,
+                                                                 config_object.epw.location['longitude'],
+                                                                 config_object.epw.location['latitude'],
+                                                                 config_object.percentiles)
+    for fut_year in config_object.future_years:
+        fut_key = str(fut_year)
+        result_data[fut_key] = {}
+        future_range = morph_utils.calc_period(int(fut_year), config_object.baseline_range)
+        for pathway in [pathway for pathway in config_object.model_pathways if pathway != "historical"]:
+            result_data[fut_key][pathway] = {}
+            for percentile in config_object.percentiles:
+                percentile_key = str(percentile)
+                morphed_data = morph_epw(config_object.epw,
+                                                         config_object.user_variables,
+                                                         config_object.baseline_range,
+                                                         future_range,
+                                                         year_model_dict,
+                                                         pathway,
+                                                         percentile)
+                morphed_data.dataframe['year'] = int(fut_year)
+                result_data[fut_key][pathway][percentile_key] = morphed_data
+                if write_file==True:
+                    morphed_data.write_to_file(os.path.join(config_object.output_directory,
+                                                            f"{fut_key}_{pathway}_{percentile_key}.epw"))
+
+
+
+    return result_data
